@@ -57,7 +57,7 @@ pub struct MapCmd {
     ///
     pub msg_id: crate::messaging::MessageId,
     ///
-    pub client_sig: super::ClientSig,
+    pub client_sig: crate::messaging::ClientSigned,
     ///
     pub origin: crate::messaging::EndUser,
 }
