@@ -59,7 +59,7 @@ pub struct SequenceCmd {
     ///
     pub msg_id: crate::messaging::MessageId,
     ///
-    pub client_sig: super::ClientSig,
+    pub client_sig: crate::messaging::ClientSigned,
     ///
     pub origin: crate::messaging::EndUser,
 }
