@@ -6,12 +6,10 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use crate::messaging::{MessageType, SectionAuthorityProvider, WireMsg};
+use crate::messaging::{DstLocation, MessageType, SectionAuthorityProvider, WireMsg};
 use crate::types::PublicKey;
-use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
 use serde::{Deserialize, Serialize};
-use xor_name::XorName;
 
 /// Messages for exchanging network info, specifically on a target section for a msg.
 #[allow(clippy::large_enum_variant)]
@@ -48,12 +46,8 @@ impl SectionInfoMsg {
         }
     }
 
-    /// serialize this Query into bytes ready to be sent over the wire.
-    pub fn serialize(
-        &self,
-        dst: XorName,
-        dst_section_pk: BlsPublicKey,
-    ) -> crate::messaging::Result<Bytes> {
-        WireMsg::serialize_section_info_msg(self, dst, dst_section_pk)
+    /// Serialize this Query into bytes ready to be sent over the wire.
+    pub fn serialize(&self, dst_location: DstLocation) -> crate::messaging::Result<Bytes> {
+        WireMsg::new_section_info_msg(self, dst_location)?.serialize()
     }
 }
