@@ -6,8 +6,6 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-//! Utilities for sn_routing messages through the network.
-
 use crate::messaging::{
     node::{Network, Peer, Section},
     DstLocation,
@@ -52,23 +50,21 @@ pub(crate) fn delivery_targets(
     }
 
     let (best_section, dg_size) = match dst {
-        DstLocation::Section(target_name) => {
-            section_candidates(target_name, our_name, section, network)?
-        }
+        DstLocation::Section { name, .. } => section_candidates(name, our_name, section, network)?,
         DstLocation::EndUser(user) => {
             section_candidates(&user.xorname, our_name, section, network)?
         }
-        DstLocation::Node(target_name) => {
-            if target_name == our_name {
+        DstLocation::Node { name, .. } => {
+            if name == our_name {
                 return Ok((Vec::new(), 0));
             }
-            if let Some(node) = get_peer(target_name, section, network) {
+            if let Some(node) = get_peer(name, section, network) {
                 return Ok((vec![node], 1));
             }
 
-            candidates(target_name, our_name, section, network)?
+            candidates(name, our_name, section, network)?
         }
-        DstLocation::DirectAndUnrouted => return Err(Error::CannotRoute),
+        DstLocation::DirectAndUnrouted(_) => return Err(Error::CannotRoute),
     };
 
     Ok((best_section, dg_size))
@@ -191,7 +187,11 @@ mod tests {
             .choose(&mut rand::thread_rng())
             .context("too few elders")?;
 
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send only to the dst node.
@@ -212,7 +212,11 @@ mod tests {
         let node_state = section_signed(&sk, node_state)?;
         assert!(section.update_member(node_state));
 
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send only to the dst node.
@@ -227,7 +231,11 @@ mod tests {
         let (our_name, section, network, _) = setup_elder()?;
 
         let dst_name = section.prefix().substituted_in(rand::random());
-        let dst = DstLocation::Section(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Section {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all our elders except us.
@@ -255,7 +263,11 @@ mod tests {
             .context("unknown section")?;
 
         let dst_name = choose_elder_name(section_auth1)?;
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send only to the dst node.
@@ -274,7 +286,11 @@ mod tests {
             .context("unknown section")?;
 
         let dst_name = section_auth1.prefix.substituted_in(rand::random());
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders in the dst section
@@ -300,7 +316,11 @@ mod tests {
             .prefix
             .pushed(false)
             .substituted_in(rand::random());
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders in the dst section
@@ -324,7 +344,11 @@ mod tests {
             .context("unknown section")?;
 
         let dst_name = section_auth1.prefix.substituted_in(rand::random());
-        let dst = DstLocation::Section(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Section {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders in the final dst section
@@ -350,7 +374,11 @@ mod tests {
             .prefix
             .pushed(false)
             .substituted_in(rand::random());
-        let dst = DstLocation::Section(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Section {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to a subset of elders in the intermediary dst section
@@ -372,7 +400,11 @@ mod tests {
         let (our_name, section, network) = setup_adult()?;
 
         let dst_name = choose_elder_name(section.authority_provider())?;
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders
@@ -387,7 +419,11 @@ mod tests {
         let (our_name, section, network) = setup_adult()?;
 
         let dst_name = section.prefix().substituted_in(rand::random());
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders
@@ -402,7 +438,11 @@ mod tests {
         let (our_name, section, network) = setup_adult()?;
 
         let dst_name = section.prefix().substituted_in(rand::random());
-        let dst = DstLocation::Section(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Section {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders
@@ -419,7 +459,11 @@ mod tests {
         let dst_name = Prefix::default()
             .pushed(true)
             .substituted_in(rand::random());
-        let dst = DstLocation::Node(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Node {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders
@@ -436,7 +480,11 @@ mod tests {
         let dst_name = Prefix::default()
             .pushed(true)
             .substituted_in(rand::random());
-        let dst = DstLocation::Section(dst_name);
+        let section_pk = section.authority_provider().section_key();
+        let dst = DstLocation::Section {
+            name: dst_name,
+            section_pk,
+        };
         let (recipients, dg_size) = delivery_targets(&dst, &our_name, &section, &network)?;
 
         // Send to all elders
