@@ -8,64 +8,70 @@
 
 use super::Core;
 use crate::messaging::{
-    node::{JoinResponse, RoutingMsg, Section, Variant},
-    DstInfo,
+    node::{JoinResponse, NodeMsg, Section},
+    DstLocation, NodeMsgAuthority, WireMsg,
 };
 use crate::routing::{
     error::Result,
-    messages::{RoutingMsgUtils, SrcAuthorityUtils},
+    messages::{NodeMsgAuthorityUtils, WireMsgUtils},
     network::NetworkUtils,
     node::Node,
     routing_api::command::Command,
     section::{SectionAuthorityProviderUtils, SectionUtils},
 };
+use bls::PublicKey as BlsPublicKey;
 use std::{cmp::Ordering, net::SocketAddr};
 
 impl Core {
     pub async fn check_for_entropy(
         &self,
-        msg: &RoutingMsg,
-        dst_info: DstInfo,
-        sender: Option<SocketAddr>,
+        node_msg: &NodeMsg,
+        msg_authority: &NodeMsgAuthority,
+        dst_location: &DstLocation,
+        sender: SocketAddr,
     ) -> Result<(Option<Command>, bool)> {
         if self.is_not_elder() {
-            // Adult nodes do need to carry out entropy checking, however the message shall always
-            // be handled.
+            // Adult nodes do need to carry out entropy checking,
+            // however the message shall always be handled.
             return Ok((None, true));
         }
 
-        let (command, shall_be_handled) = match process(&self.node, &self.section, msg, dst_info)? {
-            (Some(msg_to_send), shall_be_handled) => {
-                let command = self.relay_message(&msg_to_send).await?;
-                (command, shall_be_handled)
+        let (command, shall_be_handled) = match dst_location.section_pk() {
+            None => return Ok((None, true)),
+            Some(section_pk) => {
+                match process(
+                    &self.node,
+                    &self.section,
+                    node_msg,
+                    msg_authority,
+                    section_pk,
+                )? {
+                    (Some(msg_to_send), shall_be_handled) => {
+                        let command = self.relay_message(msg_to_send).await?;
+                        (command, shall_be_handled)
+                    }
+                    (None, shall_be_handled) => (None, shall_be_handled),
+                }
             }
-            (None, shall_be_handled) => (None, shall_be_handled),
         };
 
         if shall_be_handled || command.is_some() {
             Ok((command, shall_be_handled))
         } else {
             // For the case of receiving a JoinRequest not matching our prefix.
-            let sender_name = msg.src.name();
-            let sender_addr = if let Some(addr) = sender {
-                addr
-            } else {
-                error!("JoinRequest from {:?} without address", sender_name);
-                return Ok((None, false));
-            };
-
+            let sender_name = msg_authority.name();
             let section_auth = self
                 .network
                 .closest(&sender_name)
                 .unwrap_or_else(|| self.section.authority_provider());
 
-            let variant =
-                Variant::JoinResponse(Box::new(JoinResponse::Redirect(section_auth.clone())));
+            let node_msg =
+                NodeMsg::JoinResponse(Box::new(JoinResponse::Redirect(section_auth.clone())));
 
-            trace!("Sending {:?} to {}", variant, sender_name);
+            trace!("Sending {:?} to {}", node_msg, sender_name);
             let cmd = self.send_direct_message(
-                (sender_name, sender_addr),
-                variant,
+                (sender_name, sender),
+                node_msg,
                 section_auth.section_key(),
             )?;
 
@@ -76,55 +82,54 @@ impl Core {
 
 // On reception of an incoming message, determine the msg to send in order to
 // bring our's and the sender's knowledge about each other up to date. Returns a tuple of
-// `RoutingMsg` and `bool`. The boolean signals if the incoming message shall still be processed.
+// `NodeMsg` and `bool`. The boolean signals if the incoming message shall still be processed.
 // If entropy is found, we do not process the incoming message by returning `false`.
 fn process(
     node: &Node,
     section: &Section,
-    msg: &RoutingMsg,
-    dst_info: DstInfo,
-) -> Result<(Option<RoutingMsg>, bool)> {
-    let src_name = msg.src.name();
+    node_msg: &NodeMsg,
+    msg_authority: &NodeMsgAuthority,
+    dst_section_pk: BlsPublicKey,
+) -> Result<(Option<WireMsg>, bool)> {
+    let src_name = msg_authority.name();
+
     if section.prefix().matches(&src_name) {
         // This message is from our section. We update our members via the `Sync` message which is
         // done elsewhere.
         return Ok((None, true));
     }
 
-    let dst = msg.src.src_location().to_dst();
+    let dst = msg_authority.src_location().to_dst();
 
     if let Ordering::Less = section
         .chain()
-        .cmp_by_position(&dst_info.dst_section_pk, section.chain().last_key())
+        .cmp_by_position(&dst_section_pk, section.chain().last_key())
     {
         info!("Anti-Entropy: Source's knowledge of our key is outdated, send them an update.");
-        let chain = if let Ok(chain) = section
-            .chain()
-            .get_proof_chain_to_current(&dst_info.dst_section_pk)
-        {
+        let chain = if let Ok(chain) = section.chain().get_proof_chain_to_current(&dst_section_pk) {
             chain
         } else {
             trace!(
                 "Cannot find section_key {:?} within the chain",
-                dst_info.dst_section_pk
+                dst_section_pk
             );
             // In case a new node is trying to bootstrap from us, not being its matching section.
             // Reply with no msg and with false flag to send back a JoinResponse::Redirect.
-            if let Variant::JoinRequest(_) = msg.variant {
+            if let NodeMsg::JoinRequest(_) = node_msg {
                 return Ok((None, false));
             }
             section.chain().clone()
         };
 
         let section_auth = section.section_signed_authority_provider();
-        let variant = Variant::SectionKnowledge {
+        let node_msg = NodeMsg::SectionKnowledge {
             src_info: (section_auth.clone(), chain),
-            msg: Some(Box::new(msg.clone())),
+            msg: Some(Box::new(node_msg.clone())),
         };
-        let msg = RoutingMsg::single_src(
+        let msg = WireMsg::single_src(
             node,
             dst,
-            variant,
+            node_msg,
             section.authority_provider().section_key(),
         )?;
 
@@ -137,14 +142,14 @@ fn process(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::messaging::DstLocation;
+    use crate::messaging::{MessageType, NodeSigned};
     use crate::routing::{
         dkg::test_utils::section_signed,
         ed25519,
         section::test_utils::{gen_addr, gen_section_authority_provider},
         XorName, ELDER_SIZE, MIN_ADULT_AGE,
     };
-    use anyhow::{Context, Result};
+    use anyhow::{anyhow, Context, Result};
     use assert_matches::assert_matches;
     use secured_linked_list::SecuredLinkedList;
     use xor_name::Prefix;
@@ -153,16 +158,19 @@ mod tests {
     fn everything_up_to_date() -> Result<()> {
         let env = Env::new(1)?;
 
-        let msg = env.create_message(
+        let (node_msg, node_msg_auth) = env.create_message(
             &env.their_prefix,
             env.section.authority_provider().section_key(),
         )?;
-        let dst_info = DstInfo {
-            dst: XorName::random(),
-            dst_section_pk: *env.section.chain().last_key(),
-        };
 
-        let (msg_to_send, _) = process(&env.node, &env.section, &msg, dst_info)?;
+        let dst_section_pk = *env.section.chain().last_key();
+        let (msg_to_send, _) = process(
+            &env.node,
+            &env.section,
+            &node_msg,
+            &node_msg_auth,
+            dst_section_pk,
+        )?;
         assert_eq!(msg_to_send, None);
 
         Ok(())
@@ -175,16 +183,18 @@ mod tests {
         let our_new_sk = bls::SecretKey::random();
         let our_new_pk = our_new_sk.public_key();
 
-        let msg = env.create_message(
+        let (node_msg, node_msg_auth) = env.create_message(
             env.section.prefix(),
             env.section.authority_provider().section_key(),
         )?;
-        let dst_info = DstInfo {
-            dst: env.node.name(),
-            dst_section_pk: our_new_pk,
-        };
 
-        let (msg_to_send, _) = process(&env.node, &env.section, &msg, dst_info)?;
+        let (msg_to_send, _) = process(
+            &env.node,
+            &env.section,
+            &node_msg,
+            &node_msg_auth,
+            our_new_pk,
+        )?;
 
         assert_eq!(msg_to_send, None);
 
@@ -195,19 +205,27 @@ mod tests {
     fn outdated_dst_key_from_other_section() -> Result<()> {
         let env = Env::new(2)?;
 
-        let msg = env.create_message(
+        let (node_msg, node_msg_auth) = env.create_message(
             &env.their_prefix,
             env.section.authority_provider().section_key(),
         )?;
-        let dst_info = DstInfo {
-            dst: XorName::random(),
-            dst_section_pk: *env.section.chain().root_key(),
-        };
 
-        let (msg_to_send, _) = process(&env.node, &env.section, &msg, dst_info)?;
+        let dst_section_pk = *env.section.chain().root_key();
+        let (msg_to_send, _) = process(
+            &env.node,
+            &env.section,
+            &node_msg,
+            &node_msg_auth,
+            dst_section_pk,
+        )?;
 
-        assert_matches!(msg_to_send, Some(message) => {
-            assert_matches!(message.variant, Variant::SectionKnowledge { ref src_info, .. } => {
+        let wire_msg = msg_to_send.ok_or_else(|| anyhow!("expected an anti-entropy message"))?;
+        let msg_type = wire_msg
+            .into_message()
+            .context("failed to deserialised anti-entropy message")?;
+
+        assert_matches!(msg_type, MessageType::Node{msg,..} => {
+            assert_matches!(msg, NodeMsg::SectionKnowledge { ref src_info, .. } => {
                 assert_eq!(src_info.0.value, *env.section.authority_provider());
                 assert_eq!(src_info.1, *env.section.chain());
             });
@@ -220,16 +238,19 @@ mod tests {
     fn outdated_dst_key_from_our_section() -> Result<()> {
         let env = Env::new(2)?;
 
-        let msg = env.create_message(
+        let (node_msg, node_msg_auth) = env.create_message(
             env.section.prefix(),
             env.section.authority_provider().section_key(),
         )?;
-        let dst_info = DstInfo {
-            dst: XorName::random(),
-            dst_section_pk: *env.section.chain().root_key(),
-        };
 
-        let (msg_to_send, _) = process(&env.node, &env.section, &msg, dst_info)?;
+        let dst_section_pk = *env.section.chain().root_key();
+        let (msg_to_send, _) = process(
+            &env.node,
+            &env.section,
+            &node_msg,
+            &node_msg_auth,
+            dst_section_pk,
+        )?;
 
         assert_eq!(msg_to_send, None);
 
@@ -266,20 +287,26 @@ mod tests {
 
         fn create_message(
             &self,
-            src_section: &Prefix,
-            section_pk: bls::PublicKey,
-        ) -> Result<RoutingMsg> {
+            src_section_prefix: &Prefix,
+            section_pk: BlsPublicKey,
+        ) -> Result<(NodeMsg, NodeMsgAuthority)> {
             let sender = Node::new(
-                ed25519::gen_keypair(&src_section.range_inclusive(), MIN_ADULT_AGE),
+                ed25519::gen_keypair(&src_section_prefix.range_inclusive(), MIN_ADULT_AGE),
                 gen_addr(),
             );
 
-            Ok(RoutingMsg::single_src(
-                &sender,
-                DstLocation::Section(self.node.name()),
-                Variant::UserMessage(b"hello".to_vec()),
+            let node_msg = NodeMsg::StartConnectivityTest(XorName::random());
+            let msg_payload = WireMsg::serialize_msg_payload(&node_msg)
+                .context("Failed to create a test NodeMsg")?;
+
+            let signature = ed25519::sign(&msg_payload, &sender.keypair);
+            let node_msg_authority = NodeMsgAuthority::Node(NodeSigned {
+                public_key: sender.keypair.public,
                 section_pk,
-            )?)
+                signature,
+            });
+
+            Ok((node_msg, node_msg_authority))
         }
     }
 
