@@ -6,20 +6,21 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{Comm, Command};
+use super::{Command, Event};
 use crate::messaging::{
-    node::{
-        JoinAsRelocatedResponse, JoinRejectionReason, JoinResponse, RoutingMsg, Section,
-        SrcAuthority, Variant,
-    },
-    DstLocation, MessageType,
+    node::{NodeMsg, Section},
+    DstLocation, MsgKind, WireMsg,
 };
 use crate::routing::{
-    core::Core, error::Result, event::Event, messages::RoutingMsgUtils, node::Node,
-    peer::PeerUtils, routing_api::comm::SendStatus, section::SectionPeersUtils,
-    section::SectionUtils, Error, XorName,
+    core::{Core, SendStatus},
+    error::Result,
+    messages::WireMsgUtils,
+    node::Node,
+    peer::PeerUtils,
+    section::SectionPeersUtils,
+    section::SectionUtils,
+    Error, XorName,
 };
-use crate::types::PublicKey;
 use itertools::Itertools;
 use std::{net::SocketAddr, sync::Arc, time::Duration};
 use tokio::{
@@ -31,26 +32,26 @@ use tracing::Instrument;
 // `Command` Dispatcher.
 pub(crate) struct Dispatcher {
     pub(super) core: RwLock<Core>,
-    pub(super) comm: Comm,
 
     cancel_timer_tx: watch::Sender<bool>,
     cancel_timer_rx: watch::Receiver<bool>,
 }
 
 impl Dispatcher {
-    pub fn new(state: Core, comm: Comm) -> Self {
+    pub fn new(core: Core) -> Self {
         let (cancel_timer_tx, cancel_timer_rx) = watch::channel(false);
         Self {
-            core: RwLock::new(state),
-            comm,
+            core: RwLock::new(core),
             cancel_timer_tx,
             cancel_timer_rx,
         }
     }
 
-    /// Send provided Event to the user which shall receive it through the EventStream
-    pub async fn send_event(&self, event: Event) {
-        self.core.read().await.send_event(event).await
+    // Terminate this routing instance - cancel all scheduled timers including any future ones,
+    // close all network connections and stop accepting new connections.
+    pub async fn terminate(&self) {
+        let _ = self.cancel_timer_tx.send(true);
+        self.core.read().await.comm.terminate().await;
     }
 
     /// Handles the given command and transitively any new commands that are produced during its
@@ -64,19 +65,25 @@ impl Dispatcher {
         Ok(())
     }
 
+    // Note: this indirecton is needed. Trying to call `spawn(self.handle_commands(...))` directly
+    // inside `handle_commands` causes compile error about type check cycle.
+    fn spawn_handle_commands(self: Arc<Self>, command: Command) {
+        let _ = tokio::spawn(self.handle_commands(command));
+    }
+
     /// Handles a single command.
     pub async fn handle_command(&self, command: Command) -> Result<Vec<Command>> {
         // Create a tracing span containing info about the current node. This is very useful when
         // analyzing logs produced by running multiple nodes within the same process, for example
         // from integration tests.
         let span = {
-            let state = self.core.read().await;
+            let core = self.core.read().await;
             trace_span!(
                 "handle_command",
-                name = %state.node().name(),
-                prefix = format_args!("({:b})", state.section().prefix()),
-                age = state.node().age(),
-                elder = state.is_elder(),
+                name = %core.node().name(),
+                prefix = format_args!("({:b})", core.section().prefix()),
+                age = core.node().age(),
+                elder = core.is_elder(),
             )
         };
 
@@ -92,79 +99,15 @@ impl Dispatcher {
         .await
     }
 
-    // Terminate this routing instance - cancel all scheduled timers including any future ones,
-    // close all network connections and stop accepting new connections.
-    pub fn terminate(&self) {
-        let _ = self.cancel_timer_tx.send(true);
-        self.comm.terminate()
-    }
-
     async fn try_handle_command(&self, command: Command) -> Result<Vec<Command>> {
         match command {
-            Command::HandleMessage {
-                sender,
-                message,
-                dst_info,
-            } => {
-                if let Some(sender) = &sender {
-                    // let's then see if we need to do a reachability test
-                    let failure = match &message.variant {
-                        Variant::JoinRequest(join_request) => {
-                            // Do this check only for the initial join request
-                            if join_request.resource_proof_response.is_none()
-                                && self.comm.is_reachable(sender).await.is_err()
-                            {
-                                Some(Variant::JoinResponse(Box::new(JoinResponse::Rejected(
-                                    JoinRejectionReason::NodeNotReachable(*sender),
-                                ))))
-                            } else {
-                                None
-                            }
-                        }
-                        Variant::JoinAsRelocatedRequest(join_request) => {
-                            // Do this check only for the initial join request
-                            if join_request.relocate_payload.is_none()
-                                && self.comm.is_reachable(sender).await.is_err()
-                            {
-                                Some(Variant::JoinAsRelocatedResponse(Box::new(
-                                    JoinAsRelocatedResponse::NodeNotReachable(*sender),
-                                )))
-                            } else {
-                                None
-                            }
-                        }
-                        _ => None,
-                    };
-
-                    if let Some(variant) = failure {
-                        let section_key = *self.core.read().await.section().chain.last_key();
-                        if let SrcAuthority::Node { public_key, .. } = message.src {
-                            trace!("Sending {:?} to {}", variant, sender);
-                            return Ok(vec![self.core.read().await.send_direct_message(
-                                (XorName::from(PublicKey::from(public_key)), *sender),
-                                variant,
-                                section_key,
-                            )?]);
-                        }
-                    }
-                }
-
+            Command::HandleMessage { sender, wire_msg } => {
                 self.core
                     .write()
                     .await
-                    .handle_message(sender, message, dst_info)
+                    .handle_message(sender, wire_msg)
                     .await
             }
-            Command::HandleSectionInfoMsg {
-                sender,
-                message,
-                dst_info,
-            } => Ok(self
-                .core
-                .write()
-                .await
-                .handle_section_info_msg(sender, message, dst_info)
-                .await),
             Command::HandleTimeout(token) => self.core.write().await.handle_timeout(token),
             Command::HandleAgreement { proposal, sig } => {
                 self.core
@@ -195,22 +138,17 @@ impl Dispatcher {
                 // send to network from routing layer
                 recipients,
                 delivery_group_size,
-                message,
+                wire_msg,
             } => {
-                self.send_message(&recipients, delivery_group_size, message)
+                self.send_message(&recipients, delivery_group_size, wire_msg)
                     .await
             }
-            Command::SendUserMessage {
-                // send to network from node layer
-                itinerary,
-                content,
-                additional_proof_chain_key: _,
-            } => {
-                self.core
-                    .write()
-                    .await
-                    .send_user_message(itinerary, content)
-                    .await
+            Command::RelayMessage(wire_msg) => {
+                if let Some(cmd) = self.core.write().await.relay_message(wire_msg).await? {
+                    Ok(vec![cmd])
+                } else {
+                    Ok(vec![])
+                }
             }
             Command::ScheduleTimeout { duration, token } => Ok(self
                 .handle_schedule_timeout(duration, token)
@@ -218,7 +156,8 @@ impl Dispatcher {
                 .into_iter()
                 .collect()),
             Command::HandleRelocationComplete { node, section } => {
-                self.handle_relocation_complete(node, section).await
+                self.handle_relocation_complete(node, section).await;
+                Ok(vec![])
             }
             Command::SetJoinsAllowed(joins_allowed) => {
                 self.core.read().await.set_joins_allowed(joins_allowed)
@@ -241,12 +180,15 @@ impl Dispatcher {
                 let msg = {
                     let core = self.core.read().await;
                     let node = core.node();
-                    let section_key = *core.section().chain.last_key();
-                    RoutingMsg::single_src(
+                    let section_pk = *core.section().chain.last_key();
+                    WireMsg::single_src(
                         node,
-                        DstLocation::Section(core.node().name()),
-                        Variant::StartConnectivityTest(name),
-                        section_key,
+                        DstLocation::Section {
+                            name: core.node().name(),
+                            section_pk,
+                        },
+                        NodeMsg::StartConnectivityTest(name),
+                        section_pk,
                     )?
                 };
                 let our_name = self.core.read().await.node().name();
@@ -272,7 +214,15 @@ impl Dispatcher {
                     .get(&name)
                     .map(|member_info| member_info.peer)
                 {
-                    if self.comm.is_reachable(peer.addr()).await.is_err() {
+                    if self
+                        .core
+                        .read()
+                        .await
+                        .comm
+                        .is_reachable(peer.addr())
+                        .await
+                        .is_err()
+                    {
                         commands.push(Command::ProposeOffline(*peer.name()));
                     }
                 }
@@ -281,24 +231,24 @@ impl Dispatcher {
         }
     }
 
-    // Note: this indirecton is needed. Trying to call `spawn(self.handle_commands(...))` directly
-    // inside `handle_commands` causes compile error about type check cycle.
-    fn spawn_handle_commands(self: Arc<Self>, command: Command) {
-        let _ = tokio::spawn(self.handle_commands(command));
-    }
-
     async fn send_message(
         &self,
         recipients: &[(XorName, SocketAddr)],
         delivery_group_size: usize,
-        message: MessageType,
+        wire_msg: WireMsg,
     ) -> Result<Vec<Command>> {
-        let cmds = match message {
-            MessageType::Node { .. } | MessageType::Routing { .. } => {
+        let cmds = match wire_msg.msg_kind() {
+            MsgKind::NodeSignedMsg(_)
+            | MsgKind::NodeBlsShareSignedMsg(_)
+            | MsgKind::SectionSignedMsg(_) => {
                 let status = self
+                    .core
+                    .read()
+                    .await
                     .comm
-                    .send(recipients, delivery_group_size, message)
+                    .send(recipients, delivery_group_size, wire_msg)
                     .await?;
+
                 match status {
                     SendStatus::MinDeliveryGroupSizeReached(failed_recipients)
                     | SendStatus::MinDeliveryGroupSizeFailed(failed_recipients) => {
@@ -311,31 +261,37 @@ impl Dispatcher {
                 }
                 .map_err(|e: Error| e)?
             }
-            MessageType::Client { .. } => {
-                for recipient in recipients {
+            MsgKind::ClientMsg(_) => {
+                // TODO: send them all together without cloning WireMsg
+                // by having the send_on_existing_connection to return SendStatus
+                for (name, addr) in recipients {
                     if self
+                        .core
+                        .read()
+                        .await
                         .comm
-                        .send_on_existing_connection(*recipient, message.clone())
+                        .send_on_existing_connection(&[(*name, *addr)], wire_msg.clone())
                         .await
                         .is_err()
                     {
                         trace!(
                             "Lost connection to client {:?} when sending message {:?}",
-                            recipient,
-                            message
+                            addr,
+                            wire_msg
                         );
-                        self.send_event(Event::ClientLost(recipient.1)).await;
                     }
                 }
                 vec![]
             }
-            MessageType::SectionInfo { .. } => {
-                for recipient in recipients {
-                    let _ = self
-                        .comm
-                        .send_on_existing_connection(*recipient, message.clone())
-                        .await;
-                }
+            MsgKind::SectionInfoMsg => {
+                let _ = self
+                    .core
+                    .read()
+                    .await
+                    .comm
+                    .send_on_existing_connection(recipients, wire_msg)
+                    .await;
+
                 vec![]
             }
         };
@@ -357,25 +313,17 @@ impl Dispatcher {
         }
     }
 
-    async fn handle_relocation_complete(
-        &self,
-        new_node: Node,
-        new_section: Section,
-    ) -> Result<Vec<Command>> {
+    async fn handle_relocation_complete(&self, new_node: Node, new_section: Section) {
         let previous_name = self.core.read().await.node().name();
         let new_keypair = new_node.keypair.clone();
 
-        let mut state = self.core.write().await;
-        let event_tx = state.event_tx.clone();
-        *state = Core::new(new_node, new_section, None, event_tx);
+        let mut core = self.core.write().await;
+        *core = core.relocated(new_node, new_section).await;
 
-        state
-            .send_event(Event::Relocated {
-                previous_name,
-                new_keypair,
-            })
-            .await;
-
-        Ok(vec![])
+        core.send_event(Event::Relocated {
+            previous_name,
+            new_keypair,
+        })
+        .await;
     }
 }
